<?php
namespace um_ext\um_social_login\core;

if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

// Include Composer's autoloader
require_once um_social_login_path . 'vendor/autoload.php';

use Hybridauth\Hybridauth;
use Hybridauth\Storage\Session;
use Hybridauth\HttpClient;

/**
 * Class Social_Login_Hybridauth
 *
 * @package um_ext\um_social_login\core
 */
class Social_Login_Hybridauth {

	/**
	 * Networks
	 * @var array
	 */
	var $networks;

	/**
	 * Load settions on wp action hook
	 *
	 * @var $UM_Session
	 */
	var $UM_Session;

	/**
	 * @var array
	 */
	public $all_networks = array();

	/**
	 * @var array
	 */
	public $available_networks = array();

	/**
	 * __construct
	 */
	function __construct() {

		if ( is_admin() ) {
			return;
		}

		if ( defined( 'REST_REQUEST' ) && REST_REQUEST ) {
			return;
		}

		if ( defined( 'DOING_AJAX' ) && DOING_AJAX ) {
			return;
		}

		/**
		 * Cache compatibility
		 */
		add_action( 'after_setup_theme', function() {
			/**
			 * Load hybridauth library to specific pages e.g. /login/ or /register/
			 */
			$um_urls = apply_filters( 'um_sso_load_hybridauth', array() );

			$REQUEST_URI = $_SERVER[ 'REQUEST_URI' ];

			$has_url = false;
			foreach ( $um_urls as $url ) {
				if ( strpos( $REQUEST_URI, $url ) !== false ) {
					$has_url = true;
					break;
				}
			}

			if ( $has_url || empty( $um_urls ) ) {

				$this->UM_Session = $this->getSession();

				add_action( 'wp', array( &$this, 'wp_init' ) );
			}
		});

		add_action( 'init', array( &$this, 'init' ) );
	}

	/**
	 * WP Init
	 */
	function wp_init() {

		$this->UM_Session = $this->getSession();

	}

	/**
	 * Init
	 */
	public function init() {
		$this->all_networks       = apply_filters( 'um_social_login_networks', array() );
		$this->available_networks = UM()->Social_Login_API()->available_networks();
	}


	/**
	 * Get Session
	 * @return object
	 */
	function getSession() {

		if ( defined( 'REST_REQUEST' ) && REST_REQUEST ) {
			return;
		}

		if ( is_admin() ) {
			return;
		}

		if ( defined( 'DOING_AJAX' ) && DOING_AJAX ) {
			return;
		}

		if ( wp_doing_cron() ) {
			return;
		}

		if ( headers_sent() ) {
			return;
		}

		if ( session_id() === '' ) {
			session_start();
		}

		if ( session_status() === PHP_SESSION_NONE ) {
			// session has not started.
			session_start();
		}

		$storage = new Session();

		return $storage;
	}

	/**
	 * Connect a user with a provider account
	 *
	 * @param  string $provider Social Provider id.
	 * @param  string $callback_url Site's callback URL.
	 *
	 */
	public function connectUser( $provider = '', $callback_url = '' ) {
		$config = $this->getConfig( $provider );

		if ( isset( $config['provider'] ) ) {
			$provider = $config['provider'];
		}

		if ( ! empty( $callback_url ) ) {
			$config['callback'] = $callback_url;
		}

		$provider_class = "Hybridauth\Provider\\$provider";
		$hybridauth     = null;

		if ( class_exists( $provider_class ) ) {
			try {

				$hybridauth = new $provider_class( $config );

				if ( isset( $_REQUEST['provider'] ) ) {
					$sso_session = $this->getSession();
					$sso_session->set( 'sso_last_auth_provider', $config['provider_slug'] );
					$sso_session->set( 'sso_last_auth_page', $config['returnUrl'] );

					if ( isset( $_REQUEST['um_sso'] ) ) {
						$sso_session->set( 'um_sso', sanitize_key( $_REQUEST['um_sso'] ) );
					}

					$hybridauth->authenticate();

				}

				if ( file_exists( um_social_login_path . "includes/core/providers/{$config['provider_slug']}.php" ) ) {
					require_once um_social_login_path . "includes/core/providers/{$config['provider_slug']}.php";
				}

				$user_profile = $hybridauth->getUserProfile();

				$access_token = $hybridauth->getAccessToken();

				$user_profile = apply_filters( "um_social_login_get_user_profile__{$config['provider_slug']}", $user_profile, $hybridauth, $access_token );
				$user_profile = apply_filters( 'um_social_login_get_user_profile', $user_profile, $hybridauth, $access_token );

				// oAuth 2.
				if ( $access_token ) {

					return array(
						'hybridauth'  => $hybridauth,
						'userProfile' => $user_profile,
						'callbackUrl' => $config['callback'],
						'returnUrl'   => $config['returnUrl'],
					);

				}

				// OpenID.
				if ( $user_profile ) {

					return array(
						'hybridauth'  => $hybridauth,
						'userProfile' => $user_profile,
						'callbackUrl' => $config['callback'],
						'returnUrl'   => $config['returnUrl'],
					);

				}

				if ( ! $user_profile || ! $access_token ) {
					return array(
						'hybridauth'  => $hybridauth,
						'has_errors'  => true,
						'callbackUrl' => $config['callback'],
						'returnUrl'   => $config['returnUrl'],
					);
				}
			}

			/**
			 * Catch Curl Errors
			 *
			 * This kind of error may happen in case of:
			 *     - Internet or Network issues.
			 *     - Your server configuration is not setup correctly.
			 *
			 * The full list of curl errors that may happen can be found at http://curl.haxx.se/libcurl/c/libcurl-errors.html
			 */

			catch ( Hybridauth\Exception\HttpClientFailureException $e ) {

				$response = array(
					'Error:'       => $hybridauth->getHttpClient()->getResponseClientError(),
					'callbackUrl:' => $config['callback'],
					'returnUrl:'   => $config['returnUrl'],
				);
				$this->log_error( implode( "\n", $response ) );

				wp_die( 'Curl text error message : ' . $hybridauth->getHttpClient()->getResponseClientError(), 'UM Social Login', array( 'back_link' => true ) );
			}

			/**
			 * Catch API Requests Errors
			 *
			 * This usually happens when requesting a:
			 *     - Wrong URI or a mal-formatted http request.
			 *     - Protected resource without providing a valid access token.
			 */
			catch ( Hybridauth\Exception\HttpRequestFailedException $e ) {

				$response = array(
					'Error:'       => $hybridauth->getHttpClient()->getResponseBody(),
					'callbackUrl:' => $config['callback'],
					'returnUrl:'   => $config['returnUrl'],
				);
				$this->log_error( implode( "\n", $response ) );

				wp_die( 'Raw API Response: ' . $hybridauth->getHttpClient()->getResponseBody(), 'UM Social Login', array( 'back_link' => true ) );
			}

			/**
			 * Base PHP's exception that catches everything [else]
			 */
			catch ( \Exception $e ) {

				$response = array(
					'Error:'       => $e->getMessage(),
					'callbackUrl:' => $config['callback'],
					'returnUrl:'   => $config['returnUrl'],
				);
				$this->log_error( implode( "\n", $response ) );

				return array(
					'hybridauth'         => $hybridauth,
					'has_errors'         => true,
					'raw_error_response' => $e->getMessage(),
					'raw_error'          => $e,
					'error_message'      => '',
					'callbackUrl'        => $config['callback'],
					'returnUrl'          => $config['returnUrl'],
				);
			}
		}
	}

	/**
	 * Get Config
	 *
	 * @param string $provider Social Provider's ID.
	 *
	 * @return array $config
	 */
	public function getConfig( $provider = '' ) {

		if ( empty( $provider ) ) {
			return false;
		}

		if ( ! isset( $this->available_networks[ $provider ] ) && ! empty( $provider ) ) {
			return;
		}

		$arr_providers = array();
		$config        = array();

		foreach ( $this->available_networks as $p_key => $p_data ) {
			$arr_providers[ $p_key ] = array(
				'enabled'       => true,
				'keys'          => $this->getKeys( $p_data['opts'] ),
				'provider'      => $p_data['hybridauth_key'],
				'provider_slug' => $p_key,
			);
		}

		$config['callback']     = $this->getCallbackUrl( $provider );
		$config['returnUrl']    = $this->getReturnUrl( $provider, $config['callback'] );
		$config['redirect_uri'] = $this->getReturnUrl( $provider, $config['callback'] );

		if ( $provider ) {
			$config = array_merge( $config, $arr_providers[ $provider ] );
		} else {
			$config['providers'] = $arr_providers;
		}

		/**
		 * Example usage:
		 * 	add_filter("um_social_login_{$provider}__config", function( $config ){
		 * 		$config[
		 *          'scope' => ['r_liteprofile', ... ],
		 *          'enabled' => true,
		 *          'keys' => [
		 *                   'key' => '...',
		 *                   'secret' => '...',
		 *          ]
		 *      ],
		 * 		return $config;
		 * 	});
		 */
		return apply_filters( "um_social_login_{$provider}__config", $config );
	}

	/**
	 * Get Provider's API keys
	 *
	 * @param  array $options  API keys.
	 * @return array $arr_keys API Keys setup.
	 */
	public function getKeys( $options ) {

		$arr_keys         = array();
		$arr_options_keys = array_keys( $options );
		foreach ( $arr_options_keys as $key ) {

			if ( strrpos( $key, 'id' ) ) {
				$arr_keys['id'] = UM()->options()->get( $key );
			}

			if ( strrpos( $key, 'key' ) ) {
				$arr_keys['key'] = UM()->options()->get( $key );
			}

			if ( strrpos( $key, 'secret' ) ) {
				$arr_keys['secret'] = UM()->options()->get( $key );
			}
		}

		return $arr_keys;
	}

	/**
	 * Get current URL
	 *
	 * @return string $current_url Site's current URL excluding specific parameters.
	 */
	public function getCurrentUrl() {

		global $wp;

		$current_url = '';

		$current_url = home_url( add_query_arg( array(), $wp->request ) );

		$current_url = remove_query_arg( array( 'provider', 'state', 'code' ), $current_url );

		// Reject all file URLs from current URL.
		if ( strpos( $current_url , "/wp-content/" ) !== false ) {
			exit;
		}

		return $current_url;

	}

	/**
	 * Get Login URL
	 *
	 * @param string  $provider Social Provider's ID.
	 * @param boolean $is_shortcode determine if social connection is via Shortcode.
	 *
	 * @return string
	 */
	public function getConnectUrl( $provider = '', $is_shortcode = false ) {

		if ( defined( 'DOING_AJAX' ) && DOING_AJAX ) {
			return;
		}

		if ( isset( $_REQUEST['provider'] ) || isset( $_REQUEST['code'] ) || isset( $_REQUEST['state'] ) ) {
			return;
		}

		if ( isset( $_REQUEST['um_dynamic_sso'] ) && ! isset( $_REQUEST['err'] ) ) {
			return;
		}

		$this->UM_Session = $this->getSession();

		$current_url = add_query_arg( 'oauthWindow', 'true', '' );

		$current_url = add_query_arg( 'provider', $provider, $current_url );

		if ( $is_shortcode ) {

			$return_url  = $this->getCurrentUrl();
			$current_url = add_query_arg( 'um_sso', $is_shortcode, $current_url );
			$this->UM_Session->set( 'um_sso_has_dynamic_return_url', true );
			$this->UM_Session->set( 'um_sso_current_url', $return_url );
			$this->UM_Session->set( 'um_social_login_redirect', $return_url );

		} else {
			$this->UM_Session->set( 'um_sso_has_dynamic_return_url', null );
			if ( ( um_is_core_page( 'login' ) || um_is_core_page( 'register' ) ) && ! isset( $_REQUEST['return_provider'] ) ) {
				$this->UM_Session->set( 'um_sso', null );
			}

			$this->UM_Session->set( '_um_shortcode_id', null );

			$return_url = $this->getCurrentUrl();
			$this->UM_Session->set( 'um_sso_current_url', $return_url );
		}

		if ( ! empty( $_REQUEST['redirect_to'] ) ) {
			$this->UM_Session->set( 'um_social_login_redirect', esc_url_raw( $_REQUEST['redirect_to'] ) );
		}

		$current_url = apply_filters( 'um_social_login_connect_url', $current_url, $provider );
		$current_url = apply_filters( "um_social_login_connect_url__{$provider}", $current_url );

		return $current_url;
	}

	/**
	 * Get Disconnect Url
	 *
	 * @param  string $provider Social Provider ID.
	 * @param  string $user_id Current User's ID.
	 * @return string $current_url The current site's URL with extra parameters.
	 */
	public function getDisconnectUrl( $provider = '', $user_id = null ) {

		if ( isset( $_REQUEST['provider'] ) ) {
			return;
		}

		$current_url = add_query_arg( 'disconnect', $provider, '' );

		return $current_url;
	}

	/**
	 * Get Callback URL
	 *
	 * @param  string $provider Network provider.
	 * @param string $return_url Site's current return URL.
	 * @return string $callback_url Site's current callback URL.
	 */
	public function getCallbackUrl( $provider = '', $return_url = false ) {

		if ( um_is_core_page( 'login' ) ) {
			$callback_url = um_get_core_page( 'login' );
		} elseif ( um_is_core_page( 'register' ) ) {
			$callback_url = um_get_core_page( 'register' );
		} elseif ( um_is_core_page( 'account' ) ) {
			$callback_url = str_replace( '//social', '/social', um_get_core_page( 'account' ) . '/social/' );
		} else {
			$callback_url = um_get_core_page( 'login' );
		}

		$callback_url = add_query_arg( 'provider', $provider, $callback_url );

		$callback_url = remove_query_arg( 'err', $callback_url );

		$callback_url = remove_query_arg( 'oauthWindow', $callback_url );

		$callback_url = apply_filters( 'um_social_login_callback_url', $callback_url, $provider );

		$callback_url = apply_filters( "um_social_login_callback_url__{$provider}", $callback_url );

		return $callback_url;
	}

	/**
	 * Get return Url
	 *
	 * @param  string $provider Social Provider ID.
	 * @param  string $callback_url Site's Callback URL.
	 * @return string $return_url Return URL.
	 */
	public function getReturnUrl( $provider = '', $callback_url = '' ) {

		$sso_session = $this->getSession();

		$callback_url = $sso_session->get( 'um_sso_current_url' );

		if ( isset( $_REQUEST['redirect_to'] ) ) {
			$callback_url = esc_url_raw( $_REQUEST['redirect_to'] );
		}

		$return_url = remove_query_arg( 'provider', $callback_url );

		$has_dynamic_sso = $sso_session->get( 'um_sso' );

		if ( $has_dynamic_sso && ! um_is_core_page( 'account' ) && ! um_is_core_page( 'login' ) ) {

			$return_url = remove_query_arg( 'provider', $return_url );

			$return_url = add_query_arg( 'um_dynamic_sso', 1, $return_url );

		}

		$return_url = add_query_arg( 'return_provider', $provider, $return_url );

		return $return_url;
	}

	/**
	 * Jsdump browser console for debugging purposes
	 *
	 * @param  array $args mixed parameters to dump.
	 */
	public function jsdump( $args ) {
		echo '<script>console.log(' . json_encode( $args ) . ');</script>';
	}

	/**
	 * Log errors in a text file
	 *
	 * @param string $response object response from HybridAuth.
	 */
	public function log_error( $response ) {

		if ( ! UM()->options()->get( 'enable_debug_logs' ) ) {
			return;
		}

		update_option( 'um_sso_error_log_timestamp', current_time( 'timestamp' ) );

		$response = "\n\n" . current_time( 'mysql' ) . ":\n" . $response . "\n";
		$file     = WP_CONTENT_DIR . '/um-sso-logs-' . md5( site_url() ) . '.txt';
		$open     = fopen( $file, 'a' );
		$write = fputs( $open, $response );
		fclose( $open );
	}
}
